/// <reference types="node" />
import { AcceptedZipSources, ConstructibleArchives } from './StreamArchive';
import DMArchive from './DMArchive';
import { EventEmitter } from 'events';
import TweetArchive from './TweetArchive';
import { FavoriteArchive } from './FavoriteArchive';
import UserData from './UserData';
import AdArchive from './AdArchive';
import MediaArchive from './MediaArchive';
import { ExtendedInfoContainer, TwitterArchiveLoadOptions, BasicArchiveInfo, ArchiveSyntheticInfo } from './types/Internal';
import { PartialTweetGDPR } from './types/GDPRTweets';
import { PartialFavorite } from './types/GDPRExtended';
import { GDPRMoment } from './types/GDPRMoments';
import { ClassicPayloadDetails } from './types/ClassicPayloadIndex';
import { PartialTweet, TwitterUserDetails } from './types/ClassicTweets';
import { AccountGDPR, ProfileGDPR } from './types/GDPRAccount';
import { DMFile } from './types/GDPRDMs';
export declare type ArchiveReadState = "idle" | "reading" | "indexing" | "tweet_read" | "user_read" | "dm_read" | "extended_read" | "ready";
/** Read steps fired in `'read'` event. */
export declare type ArchiveReadStep = "zipready" | "userinfosready" | "tweetsread" | "indexready" | "willreaddm" | "willreadextended";
/**
 * Represents a full Twitter Archive. Support GDPR and classic archive.
 *
 * You can check if the archive is a GDPR archive with property `.is_gdpr`.
 *
 * Available on both GDPR and old classic archives
 * -----
 * Tweets, that are available in `.tweets`.
 * Remember that, in searchs in particular, tweets are **NOT** sorted.
 *
 * Quick user information, like screen name, bio, location, ID and account
 * creation date is available through `.user[.summary]`.
 *
 *
 * Available on GDPR archives only
 * -----
 * Favorites, mutes, blocks, followings, followers, that are stored in
 * `.favorites`, `.mutes`, `.blocks`, `.followings` and `.followers`.
 *
 * Direct messages, parsed if archive is a GDPR archive, stored in `.messages`,
 * are returned and sorted from the most older to the more recent.
 *
 * Binary data of Direct Message images can be get through `.dmImagesOf(dm_id: string)` method.
 *
 * User detailled data (screen name history, email address) is in `.user` property.
 *
 * Quick start
 * -----
 *
 * Initialize a `TwitterArchive` object with a filepath, `Blob`, `ArrayBuffer` or `Uint8Array` objects.
 *
 * Then, you could add events in order to know archive read steps.
 * Finally, wait for ready-ness of the object, and use it !
 *
 * ```ts
 * import TwitterArchive, { ArchiveReadStep } from 'twitter-archive-reader';
 *
 * // From a local file
 * const archive = new TwitterArchive('filepath.zip');
 * // or file input (browser)
 * const archive = new TwitterArchive(document.querySelector('input[type="file"]').files[0]);
 *
 * // Listen for archive steps to give feedback to user
 * archive.events.on('read', ({ step }: { step: ArchiveReadStep }) => {
 *  console.log("Archive reading step:", step);
 * });
 *
 * // Wait for ready-ness
 * await archive.ready();
 * console.log("Archive is ready !");
 * ```
 */
export declare class TwitterArchive {
    protected _ready: Promise<void>;
    protected archive: ConstructibleArchives;
    /** Current archive load state. */
    state: ArchiveReadState;
    /** Where events about archive read are emitted. */
    readonly events: EventEmitter;
    protected _tweets: TweetArchive;
    protected _messages: DMArchive;
    protected _favorites: FavoriteArchive;
    protected extended_info_container: ExtendedInfoContainer;
    protected _user: UserData;
    protected _ads: AdArchive;
    protected _medias: MediaArchive;
    protected _is_gdpr: boolean;
    protected _created_at: string;
    /**
     * Twitter Archive constructor.
     *
     * Don't forget to await the archive ready-ness with `.ready()` method !
     *
     * ```ts
     * // From a local file
     * const archive = new TwitterArchive('filepath.zip');
     * // or file input (browser)
     * const archive = new TwitterArchive(document.querySelector('input[type="file"]').files[0]);
     *
     * await archive.ready();
     * ```
     *
     * @param file Archive to load.
     *
     * If you want to build an archive instance **without** a file, you can pass `null` here.
     * You must then load parts of the archive with `.loadArchivePart()` or `.loadClassicArchivePart()` !
     *
     * @param options.build_ad_archive
     * `true` if ad data should be parsed and loaded.
     *
     * If you want to save time and memory at construct time, omit this parameter, it's set to `false` by default.
     *
     * Then, before first accessing `.ads`, load the archive data present in the archive
     * using `.loadArchivePart({ current_ad_archive: true })`.
     */
    constructor(file: AcceptedZipSources | Promise<AcceptedZipSources> | null, options?: TwitterArchiveLoadOptions);
    protected initGDPR(build_ad_archive: boolean): Promise<void>;
    protected initExtendedGDPR(): Promise<void>;
    protected initClassic(): Promise<void>;
    /** ----------------- */
    /** ARCHIVE ACCESSORS */
    /** ----------------- */
    /** `true` if ZIP file is still loaded inside this instance.
     * Can be freed (to save memory, f.e.) with `.releaseZip()`. */
    get is_zip_loaded(): boolean;
    /**
     * `true` if `medias.get()` function is accessible.
     *
     * `false` if your archive does not supports medias.
     *
     * @deprecated use `.medias.has_medias` instead.
     */
    get is_dm_images_available(): boolean;
    /**
     * Archive creation date.
     *
     * **Warning**: This is accurate only for classic archives (< 2018, tweet only).
     *
     * In GDPR archives, this will be the current date.
     *
     */
    get generation_date(): Date;
    /**
     * Archive quick information.
     *
     * - `.info.archive` : `{ created_at: string, tweets: number }`
     * - `.info.user`: See `TwitterUserDetails`
     */
    get info(): BasicArchiveInfo;
    /** `true` if archive is a GDPR archive. */
    get is_gdpr(): boolean;
    /**
     * Raw archive object. Can be used to get specific files.
     *
     * Returns twitter_archive.
     */
    get raw(): ConstructibleArchives;
    /** Resolved when archive read is over. */
    ready(): Promise<void>;
    /** Access to the `DMArchive` object. Will be undefined if `.is_gdpr === false`. */
    get messages(): DMArchive;
    /** Access to the `TweetArchive` object. Contains all the tweets of this archive. */
    get tweets(): TweetArchive;
    /**
     * Access to the `FavoriteArchive` object. Contains all the favorited tweets of the archive.
     *
     * If `.is_gdpr === false`, this container will be empty.
     */
    get favorites(): FavoriteArchive;
    /**
     * Access to a set of followers IDs.
     *
     * If `.is_gdpr === false`, this container will be empty.
     */
    get followers(): Set<string>;
    /**
     * Access to a set of followings user IDs.
     *
     * If `.is_gdpr === false`, this container will be empty.
     */
    get followings(): Set<string>;
    /**
     * Access to a set of blocked user IDs.
     *
     * If `.is_gdpr === false`, this container will be empty.
     */
    get blocks(): Set<string>;
    /**
     * Access to a set of muted user IDs.
     *
     * If `.is_gdpr === false`, this container will be empty.
     */
    get mutes(): Set<string>;
    /**
     * Access to archive Twitter moments.
     *
     * If `.is_gdpr === false`, this array will be empty.
     */
    get moments(): GDPRMoment[];
    /**
     * Access to your subscribed and created lists, and the lists you were added into.
     *
     * If `.is_gdpr === false`, this container will be contain empty arrays.
     */
    get lists(): {
        created: string[];
        member_of: string[];
        subscribed: string[];
    };
    /**
     * All the archive owner's user data on Twitter.
     *
     * Contains data sended to Twitter about archive owner, like:
     * - **Screen name (@)**
     * - **Tweet name (TN)**
     * - **Account creation date**
     * - **Biography**
     * - **Phone number**
     * - **Personnalization** (inferred informations about user interests)
     * - **Email addresses**
     * - *...*
     *
     * If `.is_gdpr === false`, this is will contain only basic data (related to `.user.summary`).
     */
    get user(): UserData;
    /**
     * Informations about which ads archive owner seen or interacted with.
     *
     * `AdArchive` container does not contain any data if you
     * haven't used `{ build_ad_archive: true }` constructor parameter.
     */
    get ads(): AdArchive;
    /**
     * Access to medias stored in this archive, like dm images, tweet medias and profile pictures.
     */
    get medias(): MediaArchive;
    /** -------------------- */
    /** SIDELOADING MANUALLY */
    /** -------------------- */
    /**
     * Load a part of a GDPR archive.
     *
     * Set current archive as GDPR archive.
     *
     * ---
     *
     * **Warning**: The DMs image parameters / current_ad_archive causes the read of new data.
     * Read is asynchronous, you **must** wait read end before trying to get images / ad data.
     *
     * ---
     *
     * If you use the `current_*` parameters, ensure that archive is still loaded (`.is_zip_loaded`).
     */
    loadArchivePart(parts?: {
        tweets?: PartialTweetGDPR[];
        account?: AccountGDPR;
        profile?: ProfileGDPR;
        dms?: DMFile[];
        current_ad_archive?: boolean;
        favorites?: PartialFavorite[];
        blocks?: string[];
        mutes?: string[];
        followers?: string[];
        followings?: string[];
        moments?: GDPRMoment[];
    }): Promise<void>;
    /**
     * Load a part of a classic archive.
     *
     * Set current archive as non-gdpr archive.
     */
    loadClassicArchivePart(parts?: {
        tweets?: PartialTweet[];
        payload?: ClassicPayloadDetails;
        user?: TwitterUserDetails;
    }): void;
    protected initUserCache(): void;
    /** --------------------------------------- */
    /** ARCHIVE FINGERPRINTING AND INFORMATIONS */
    /** --------------------------------------- */
    /**
     * Unload ZIP file inside this instance.
     * Returns `true` if ZIP has been unloaded.
     *
     * ZIP can still be loaded in `archive.medias`, you can use its own `.releaseZip()` method.
     */
    releaseZip(): boolean;
    /**
     * Résumé of this archive.
     *
     * Contains the 'archive index' (without the tweets),
     * if the archive is GDPR, last tweet date,
     * tweet count and dm count.
     */
    get synthetic_info(): ArchiveSyntheticInfo;
    /**
     * `.synthetic_info`, but without hash. Used to... calculate hash.
     */
    protected get synthetic_info_without_hash(): ArchiveSyntheticInfo;
    /**
     * Create a pseudo-unique hash from a archive information (number of tweets, owner, last tweet date...),
     * in order to have a fingerprint of one archive.
     */
    static hash(from: ArchiveSyntheticInfo | TwitterArchive): string;
    /**
     * Pseudo-unique hash from the archive information (number of tweets, owner, last tweet date...),
     * in order to have a fingerprint of this archive.
     */
    get hash(): string;
}
