"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = __importDefault(require("./index"));
const jszip_1 = __importDefault(require("jszip"));
function convertConversationToGDPRConversation(conversation) {
    let first = true;
    const msgs = [];
    function addEvents(e) {
        for (const [key, vals] of Object.entries(e)) {
            for (const val of vals) {
                msgs.push({ [key]: val });
            }
        }
    }
    for (const msg of conversation.all) {
        if (first) {
            first = false;
            if (msg.events && msg.events.before) {
                addEvents(msg.events.before);
            }
        }
        msgs.push({ messageCreate: {
                recipientId: msg.recipientId,
                createdAt: msg.createdAt,
                mediaUrls: msg.mediaUrls,
                text: msg.text,
                senderId: msg.senderId,
                id: msg.id
            } });
        if (msg.events && msg.events.after) {
            addEvents(msg.events.after);
        }
    }
    return {
        dmConversation: {
            conversationId: conversation.id,
            messages: msgs
        }
    };
}
function isGdprSNHArray(array) {
    if (array.length) {
        return 'screenNameChange' in array[0];
    }
    return false;
}
class ArchiveSaver {
    /**
     * Create a save from a Twitter Archive.
     *
     * Restore an `ArchiveSave` with `.restore()`.
     *
     * Default parameter for {options} is:
     * ```ts
     * options = {
     *  tweets: true,
     *  dms: true,
     *  mutes: true,
     *  favorites: true,
     *  blocks: true,
     *  user: {}
     * }
     * ```
     */
    static async create(archive, options = {
        tweets: true,
        dms: true,
        mutes: true,
        favorites: true,
        blocks: true,
        user: {},
    }) {
        const info = archive.synthetic_info;
        let tweet_zip;
        if (options.tweets) {
            const tweets = archive.tweets.all;
            for (const tweet of tweets) {
                delete tweet.created_at_d;
            }
            tweet_zip = await new jszip_1.default().file("tweet.json", JSON.stringify(tweets)).generateAsync({
                type: "arraybuffer",
                compression: "DEFLATE",
                compressionOptions: {
                    level: 6 // Not too much, if we want a good generation time
                }
            });
        }
        const mutes = options.mutes ? [...archive.mutes] : [];
        const blocks = options.blocks ? [...archive.blocks] : [];
        let dms = null;
        if (options.dms && archive.is_gdpr && archive.messages) {
            // Swallow copy all the dms, save them to a JSZip instance
            /*
              dm.json => [
                GDPRConversation,
                ...
              ]
            */
            dms = await new jszip_1.default()
                .file("dm.json", JSON.stringify(archive.messages.all.map(convertConversationToGDPRConversation)))
                .generateAsync({
                type: "arraybuffer",
                compression: "DEFLATE",
                compressionOptions: {
                    level: 6 // Not too much, if we want a good generation time
                }
            });
        }
        info.version = this.CURRENT_EXPORT_VERSION;
        let ads = null;
        if (options.ad_archive) {
            const all = JSON.stringify({
                impressions: archive.ads.impressions,
                engagements: archive.ads.engagements,
                mobile_conversions: archive.ads.mobile_conversions,
                online_conversions: archive.ads.online_conversions,
            });
            ads = await new jszip_1.default()
                .file("ads.json", all)
                .generateAsync({
                type: "arraybuffer",
                compression: "DEFLATE",
                compressionOptions: {
                    level: 6 // Not too much, if we want a good generation time
                }
            });
        }
        const save = {
            tweets: tweet_zip,
            dms,
            info,
            mutes,
            blocks,
            followers: options.followers ? [...archive.followers] : undefined,
            followings: options.followings ? [...archive.followings] : undefined,
            moments: options.moments ? archive.moments : undefined,
            lists: options.lists ? archive.lists : undefined,
            ad_archive: ads,
            screen_name_history: archive.user.screen_name_history,
            favorites: options.favorites ? archive.favorites.all : [],
            user: {},
        };
        // Userdata ok
        if (options.user && Object.keys(options.user).length) {
            for (const [name, value] of Object.entries(archive.user.dump())) {
                if (value && name in options.user) {
                    // @ts-ignore
                    save.user[name] = value;
                }
            }
        }
        return save;
    }
    /**
     * Create a Twitter Archive from an `ArchiveSave`.
     */
    static async restore(save) {
        save = await save;
        if (!this.SUPPORTED_SAVE_VERSIONS.includes(save.info.version)) {
            throw new Error("Save version is not supported.");
        }
        const archive = new index_1.default(null);
        const save_info = save.info;
        if (save.info.version === "1.0.0" && 'index' in save_info) {
            // @ts-ignore
            archive.loadClassicArchivePart({ user: save_info.index.info });
        }
        else {
            archive.loadClassicArchivePart({ user: save_info.info.user });
        }
        if (save.tweets) {
            const tweet_archive = await jszip_1.default.loadAsync(save.tweets);
            let current_load_object = JSON.parse(await tweet_archive.file("tweet.json").async("text"));
            // Tweets are extracted from a previous archive, they've been converted to classic format.
            archive.loadClassicArchivePart({ tweets: current_load_object });
        }
        if (save.info.is_gdpr) {
            // Side effect of this method is to define archive to GDPR format
            await archive.loadArchivePart();
        }
        if (save.dms) {
            const dm_archive = await jszip_1.default.loadAsync(save.dms);
            let current_load_object = JSON.parse(await dm_archive.file("dm.json").async("text"));
            await archive.loadArchivePart({
                dms: [current_load_object]
            });
        }
        if (save.mutes && save.mutes.length) {
            await archive.loadArchivePart({
                mutes: save.mutes,
            });
        }
        if (save.blocks && save.blocks.length) {
            await archive.loadArchivePart({
                blocks: save.blocks,
            });
        }
        if (save.followers && save.followers.length) {
            await archive.loadArchivePart({
                followers: save.followers,
            });
        }
        if (save.followings && save.followings.length) {
            await archive.loadArchivePart({
                followings: save.followings,
            });
        }
        if (save.moments && save.moments.length) {
            await archive.loadArchivePart({
                moments: save.moments,
            });
        }
        if (save.lists) {
            archive.lists.created = save.lists.created;
            archive.lists.member_of = save.lists.member_of;
            archive.lists.subscribed = save.lists.subscribed;
        }
        if (save.ad_archive) {
            const ad_archive = await jszip_1.default.loadAsync(save.ad_archive);
            let current_load_object = JSON.parse(await ad_archive.file("ads.json").async("text"));
            archive.ads.impressions = current_load_object.impressions;
            archive.ads.engagements = current_load_object.engagements;
            archive.ads.online_conversions = current_load_object.online_conversions;
            archive.ads.mobile_conversions = current_load_object.mobile_conversions;
        }
        if (save.user) {
            archive.user.loadPart(save.user);
        }
        if (archive.is_gdpr) {
            if (save.favorites) {
                archive.favorites.add(save.favorites);
            }
            // Sideload screen name history
            const sn_h = save.screen_name_history;
            if (isGdprSNHArray(sn_h)) {
                archive.user.loadPart({
                    screen_name_history: sn_h.map(e => e.screenNameChange)
                });
            }
            else {
                archive.user.loadPart({
                    screen_name_history: sn_h
                });
            }
        }
        return archive;
    }
}
exports.ArchiveSaver = ArchiveSaver;
ArchiveSaver.SUPPORTED_SAVE_VERSIONS = ["1.0.0", "1.1.0"];
ArchiveSaver.CURRENT_EXPORT_VERSION = "1.1.0";
exports.default = ArchiveSaver;
