import { AcceptedZipSources, ConstructibleArchives } from "./StreamArchive";
import UserData from "./UserData";
import { DirectMessage } from "./types/GDPRDMs";
import { PartialTweet, PartialTweetMediaEntity } from "./types/ClassicTweets";
import { MediaGDPREntity } from "./types/GDPRTweets";
export declare type ArchiveDMImagesFormation = "none" | "inside" | "zipped";
declare type ExisitingArchives = "dm_single" | "dm_group" | "tweet" | "moment" | "profile";
export declare enum MediaArchiveType {
    SingleDM = "single-dm",
    GroupDM = "group-dm",
    Moment = "moment",
    Tweet = "tweet",
    Profile = "profile"
}
export declare class MediaArchive {
    protected archive: ConstructibleArchives;
    protected dm_single: SingleMediaArchive | undefined;
    protected dm_group: SingleMediaArchive | undefined;
    protected tweet: SingleMediaArchive | undefined;
    protected profile: SingleMediaArchive | undefined;
    protected moment: SingleMediaArchive | undefined;
    protected static readonly DM_SINGLE_FOLDER = "direct_message_media";
    protected static readonly DM_GROUP_FOLDER = "direct_message_group_media";
    protected static readonly TWEET_FOLDER = "tweet_media";
    protected static readonly PROFILE_FOLDER = "profile_media";
    protected static readonly MOMENT_FOLDER = "moments_media";
    protected _store_type: ArchiveDMImagesFormation;
    constructor(archive: ConstructibleArchives);
    /**
    * Get a media from a specific media type.
    *
    * @param from Media archive
    * @param name Filename (exact filename required)
    * @param as_array_buffer True if return type is ArrayBuffer. Otherwise, Blob will be used.
    * By default, returns ArrayBuffer on Node.js and Blob when available.
    */
    get(from: MediaArchiveType, name: string, as_array_buffer?: boolean): Promise<ArrayBuffer>;
    /**
    * List files available on a specific media archive.
    *
    * @param of_archive Media archive
    */
    list(of_archive: MediaArchiveType): Promise<string[]>;
    /**
     * Return all the images of a direct message, as blob or array buffer.
     *
     * If the message does not exists or the DM archive is not loaded / available,
     * return an empty array.
     *
     * Otherwise, return a array of `Blob` / `ArrayBuffer`
     *
     * @param direct_message Direct message object
     * @param as_array_buffer Return an `ArrayBuffer` array, instead of a `Blob` array
     */
    ofDm(direct_message: DirectMessage, as_array_buffer?: boolean): Promise<(Blob | ArrayBuffer)[]>;
    /**
     * Extract the related media file to a URL present in the `mediaUrls` array of a Direct Message.
     */
    fromDmMediaUrl(url: string, is_group?: boolean, as_array_buffer?: boolean): Promise<ArrayBuffer>;
    /**
     * Get all the medias related to a tweet.
     */
    ofTweet(tweet: PartialTweet, as_array_buffer?: boolean): Promise<(Blob | ArrayBuffer)[]>;
    /**
     * Extract related tweet video or picture from a media entity.
     *
     * @throws If not valid media found, promise is rejected.
     *
     * ```ts
     * const tweet = archive.tweets.all[0];
     *
     * if (tweet.extended_entities || tweet.entities) {
     *    // Always try to use extended entities instead of classic entities
     *    const m_entities = (tweet.extended_entities || tweet.entities).media;
     *
     *    if (m_entities && m_entities.length) {
     *      const media_file = archive.medias.fromTweetMediaEntity(m_entities[0]);
     *    }
     * }
     * ```
     */
    fromTweetMediaEntity(media_entity: MediaGDPREntity | PartialTweetMediaEntity, as_array_buffer?: boolean): Promise<ArrayBuffer>;
    /**
     * Get the profile banner of given user.
     *
     * The first parameter should generally be `archive.user`.
     *
     * If user has no banner, this method returns `Promise<void>`.
     */
    getProfileBannerOf(user: UserData, as_array_buffer?: boolean): Promise<Blob | ArrayBuffer>;
    /**
     * Get the profile picture of given user.
     *
     * The first parameter should generally be `archive.user`.
     *
     * If user has no profile picture, this method returns `Promise<void>`.
     */
    getProfilePictureOf(user: UserData, as_array_buffer?: boolean): Promise<Blob | ArrayBuffer>;
    /**
     * Extract a moment header image from GDPR archive (exact filename required).
     *
     * In order to have tweets medias inside the moments (duplicated by Twitter in the archive, use `.ofTweet(tweet)`).
     *
     * Shortcut of `.get(MediaArchiveType.Moment, name, as_array_buffer)`, prefer using this instead.
     *
     * @param name Media filename
     */
    fromMomentDirectory(name: string, as_array_buffer?: boolean): Promise<Blob | ArrayBuffer>;
    get is_medias_zipped(): boolean;
    get has_medias(): boolean;
    get is_zip_loaded(): boolean;
    releaseZip(): void;
    loadArchive(parts: {
        dm_single_archive?: AcceptedZipSources | Promise<AcceptedZipSources>;
        dm_group_archive?: AcceptedZipSources | Promise<AcceptedZipSources>;
        moment_archive?: AcceptedZipSources | Promise<AcceptedZipSources>;
        tweet_archive?: AcceptedZipSources | Promise<AcceptedZipSources>;
        profile_archive?: AcceptedZipSources | Promise<AcceptedZipSources>;
    }): Promise<void>;
    protected initOrAwaitArchive(archive_type: ExisitingArchives, init_folder: string): Promise<void>;
    protected static autoDetectStoreType(archive: ConstructibleArchives): ArchiveDMImagesFormation;
}
export default MediaArchive;
export declare class SingleMediaArchive {
    protected _archive: ConstructibleArchives | undefined;
    protected _ready: Promise<void>;
    protected _ok: boolean;
    constructor(full_archive: ConstructibleArchives | null, dir_name: string);
    get archive(): ConstructibleArchives;
    get ok(): boolean;
    get ready(): Promise<void>;
    get files(): string[];
    file<T = true>(name: string, as_array_buffer: T): Promise<ArrayBuffer>;
    file<T = false>(name: string, as_array_buffer: T): Promise<Blob>;
    sideload(archive: ConstructibleArchives): Promise<void>;
    protected static autoDetectIfArrayBuffer(): boolean;
}
