"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const StreamArchive_1 = require("./StreamArchive");
var MediaArchiveType;
(function (MediaArchiveType) {
    MediaArchiveType["SingleDM"] = "single-dm";
    MediaArchiveType["GroupDM"] = "group-dm";
    MediaArchiveType["Moment"] = "moment";
    MediaArchiveType["Tweet"] = "tweet";
    MediaArchiveType["Profile"] = "profile";
})(MediaArchiveType = exports.MediaArchiveType || (exports.MediaArchiveType = {}));
class MediaArchive {
    constructor(archive) {
        this.archive = archive;
        this._store_type = "none";
        if (this.archive)
            this._store_type = MediaArchive.autoDetectStoreType(this.archive);
    }
    /*
     * MEDIA GETTERS: PUBLIC
     */
    /**
    * Get a media from a specific media type.
    *
    * @param from Media archive
    * @param name Filename (exact filename required)
    * @param as_array_buffer True if return type is ArrayBuffer. Otherwise, Blob will be used.
    * By default, returns ArrayBuffer on Node.js and Blob when available.
    */
    async get(from, name, as_array_buffer) {
        switch (from) {
            case MediaArchiveType.SingleDM: {
                await this.initOrAwaitArchive("dm_single", MediaArchive.DM_SINGLE_FOLDER);
                return this.dm_single.file(name, as_array_buffer);
            }
            case MediaArchiveType.GroupDM: {
                await this.initOrAwaitArchive("dm_group", MediaArchive.DM_GROUP_FOLDER);
                return this.dm_group.file(name, as_array_buffer);
            }
            case MediaArchiveType.Tweet: {
                await this.initOrAwaitArchive("tweet", MediaArchive.TWEET_FOLDER);
                return this.tweet.file(name, as_array_buffer);
            }
            case MediaArchiveType.Moment: {
                await this.initOrAwaitArchive("moment", MediaArchive.MOMENT_FOLDER);
                return this.moment.file(name, as_array_buffer);
            }
            case MediaArchiveType.Profile: {
                await this.initOrAwaitArchive("profile", MediaArchive.PROFILE_FOLDER);
                return this.profile.file(name, as_array_buffer);
            }
        }
        throw new Error("Unsupported media type.");
    }
    /**
    * List files available on a specific media archive.
    *
    * @param of_archive Media archive
    */
    async list(of_archive) {
        switch (of_archive) {
            case MediaArchiveType.SingleDM: {
                await this.initOrAwaitArchive("dm_single", MediaArchive.DM_SINGLE_FOLDER);
                return this.dm_single.files;
            }
            case MediaArchiveType.GroupDM: {
                await this.initOrAwaitArchive("dm_group", MediaArchive.DM_GROUP_FOLDER);
                return this.dm_group.files;
            }
            case MediaArchiveType.Tweet: {
                await this.initOrAwaitArchive("tweet", MediaArchive.TWEET_FOLDER);
                return this.tweet.files;
            }
            case MediaArchiveType.Moment: {
                await this.initOrAwaitArchive("moment", MediaArchive.MOMENT_FOLDER);
                return this.moment.files;
            }
            case MediaArchiveType.Profile: {
                await this.initOrAwaitArchive("profile", MediaArchive.PROFILE_FOLDER);
                return this.profile.files;
            }
        }
        throw new Error("Unsupported media type.");
    }
    /*
     * MEDIA GETTERS: PRIVATE
     */
    // -------------------
    // - Direct Messages -
    // -------------------
    /**
     * Return all the images of a direct message, as blob or array buffer.
     *
     * If the message does not exists or the DM archive is not loaded / available,
     * return an empty array.
     *
     * Otherwise, return a array of `Blob` / `ArrayBuffer`
     *
     * @param direct_message Direct message object
     * @param as_array_buffer Return an `ArrayBuffer` array, instead of a `Blob` array
     */
    async ofDm(direct_message, as_array_buffer) {
        const images = [];
        const is_group = !direct_message.recipientId || direct_message.recipientId === "0";
        for (const media of direct_message.mediaUrls) {
            images.push(this.fromDmMediaUrl(media, is_group, as_array_buffer));
        }
        return Promise.all(images);
    }
    /**
     * Extract the related media file to a URL present in the `mediaUrls` array of a Direct Message.
     */
    fromDmMediaUrl(url, is_group = false, as_array_buffer) {
        const [, , , , id, , image] = url.split('/');
        if (id && image) {
            if (is_group)
                return this.get(MediaArchiveType.GroupDM, id + "-" + image, as_array_buffer);
            else
                return this.get(MediaArchiveType.SingleDM, id + "-" + image, as_array_buffer);
        }
        return Promise.reject("URL is invalid");
    }
    // ----------
    // - Tweets -
    // ----------
    /**
     * Get all the medias related to a tweet.
     */
    async ofTweet(tweet, as_array_buffer) {
        const entities = tweet.extended_entities;
        if (!entities || !entities.media) {
            return [];
        }
        const medias = [];
        for (const media of entities.media) {
            medias.push(this.fromTweetMediaEntity(media, as_array_buffer));
        }
        return Promise.all(medias);
    }
    /**
     * Extract related tweet video or picture from a media entity.
     *
     * @throws If not valid media found, promise is rejected.
     *
     * ```ts
     * const tweet = archive.tweets.all[0];
     *
     * if (tweet.extended_entities || tweet.entities) {
     *    // Always try to use extended entities instead of classic entities
     *    const m_entities = (tweet.extended_entities || tweet.entities).media;
     *
     *    if (m_entities && m_entities.length) {
     *      const media_file = archive.medias.fromTweetMediaEntity(m_entities[0]);
     *    }
     * }
     * ```
     */
    async fromTweetMediaEntity(media_entity, as_array_buffer) {
        if ('video_info' in media_entity) {
            // This is a gif or a video
            // Find the best variant
            const mp4s = media_entity.video_info.variants.filter(v => v.content_type === "video/mp4").filter(v => v.bitrate);
            if (mp4s) {
                const better = mp4s.sort((a, b) => Number(b.bitrate) - Number(a.bitrate))[0];
                const url = better.url.split('/').pop();
                const url_without_qs = url.split('?')[0];
                if (url_without_qs) {
                    return this.get(MediaArchiveType.Tweet, url_without_qs, as_array_buffer);
                }
            }
        }
        const url = media_entity.media_url_https.split('/').pop();
        const url_without_qs = url.split('?')[0];
        if (url_without_qs) {
            return this.get(MediaArchiveType.Tweet, url_without_qs, as_array_buffer);
        }
        throw new Error("No valid file in this media entity.");
    }
    // -----------
    // - Profile -
    // -----------
    /**
     * Get the profile banner of given user.
     *
     * The first parameter should generally be `archive.user`.
     *
     * If user has no banner, this method returns `Promise<void>`.
     */
    async getProfileBannerOf(user, as_array_buffer) {
        if (user.profile_banner_url) {
            const img_name = user.profile_banner_url.split('/').pop();
            if (img_name) {
                return this.get(MediaArchiveType.Profile, img_name, as_array_buffer);
            }
        }
    }
    /**
     * Get the profile picture of given user.
     *
     * The first parameter should generally be `archive.user`.
     *
     * If user has no profile picture, this method returns `Promise<void>`.
     */
    async getProfilePictureOf(user, as_array_buffer) {
        if (user.profile_img_url) {
            const img_name = user.profile_img_url.split('/').pop();
            if (img_name) {
                return this.get(MediaArchiveType.Profile, img_name, as_array_buffer);
            }
        }
    }
    // -----------
    // - Moments -
    // -----------
    /**
     * Extract a moment header image from GDPR archive (exact filename required).
     *
     * In order to have tweets medias inside the moments (duplicated by Twitter in the archive, use `.ofTweet(tweet)`).
     *
     * Shortcut of `.get(MediaArchiveType.Moment, name, as_array_buffer)`, prefer using this instead.
     *
     * @param name Media filename
     */
    async fromMomentDirectory(name, as_array_buffer) {
        return this.get(MediaArchiveType.Moment, name, as_array_buffer);
    }
    /*
     * ACCESSORS
     */
    get is_medias_zipped() {
        return this._store_type === "zipped";
    }
    get has_medias() {
        return this._store_type !== "none";
    }
    get is_zip_loaded() {
        return !!this.archive;
    }
    releaseZip() {
        this.archive = undefined;
    }
    /*
     * INIT ARCHIVES
     */
    async loadArchive(parts) {
        if (parts.dm_group_archive) {
            this.dm_group = new SingleMediaArchive(null, "");
            await this.dm_group.sideload(StreamArchive_1.constructArchive(await parts.dm_group_archive));
        }
        if (parts.dm_single_archive) {
            this.dm_single = new SingleMediaArchive(null, "");
            await this.dm_single.sideload(StreamArchive_1.constructArchive(await parts.dm_single_archive));
        }
        if (parts.moment_archive) {
            this.moment = new SingleMediaArchive(null, "");
            await this.moment.sideload(StreamArchive_1.constructArchive(await parts.moment_archive));
        }
        if (parts.tweet_archive) {
            this.tweet = new SingleMediaArchive(null, "");
            await this.tweet.sideload(StreamArchive_1.constructArchive(await parts.tweet_archive));
        }
        if (parts.profile_archive) {
            this.profile = new SingleMediaArchive(null, "");
            await this.profile.sideload(StreamArchive_1.constructArchive(await parts.profile_archive));
        }
    }
    initOrAwaitArchive(archive_type, init_folder) {
        if (!this[archive_type]) {
            this[archive_type] = new SingleMediaArchive(this.archive, init_folder);
        }
        return this[archive_type].ready;
    }
    /*
     * HELPERS
     */
    static autoDetectStoreType(archive) {
        if (archive.searchDir(/direct_message_media/).length) {
            const folder = archive.dir('direct_message_media');
            const query = folder.search(/\.zip$/);
            if (query.length) {
                return "zipped";
            }
            return "inside";
        }
        if (archive.searchDir(/direct_message_group_media/).length) {
            const folder = archive.dir('direct_message_group_media');
            const query = folder.search(/\.zip$/);
            if (query.length) {
                return "zipped";
            }
            return "inside";
        }
        if (archive.searchDir(/tweet_media/).length) {
            const folder = archive.dir('tweet_media');
            const query = folder.search(/\.zip$/);
            if (query.length) {
                return "zipped";
            }
            return "inside";
        }
        if (archive.searchDir(/profile_media/).length) {
            const folder = archive.dir('profile_media');
            const query = folder.search(/\.zip$/);
            if (query.length) {
                return "zipped";
            }
            return "inside";
        }
        return "none";
    }
}
exports.MediaArchive = MediaArchive;
MediaArchive.DM_SINGLE_FOLDER = "direct_message_media";
MediaArchive.DM_GROUP_FOLDER = "direct_message_group_media";
MediaArchive.TWEET_FOLDER = "tweet_media";
MediaArchive.PROFILE_FOLDER = "profile_media";
MediaArchive.MOMENT_FOLDER = "moments_media";
exports.default = MediaArchive;
class SingleMediaArchive {
    constructor(full_archive, dir_name) {
        this._ok = false;
        if (full_archive === null) {
            this._ready = Promise.resolve();
            this._ok = true;
        }
        else {
            if (!full_archive) {
                throw new Error("Archive is not loaded. This is required to load new medias.");
            }
            this._ready = (async () => {
                const folder = full_archive.dir(dir_name);
                const query = folder.search(/\.zip$/);
                if (query.length) {
                    this._archive = await folder.fromFile(query[0]);
                    await this._archive.ready();
                }
                else {
                    this._archive = full_archive.dir(dir_name);
                }
                this._ok = true;
            })();
        }
    }
    get archive() {
        return this._archive;
    }
    get ok() {
        return this._ok;
    }
    get ready() {
        return this._ready;
    }
    get files() {
        return Object.keys(this._archive.ls(true)).filter(e => e);
    }
    async file(name, as_array_buffer) {
        if (!this._archive || !this._ok) {
            throw new Error("Archive is not loaded or hasn't been initialized properly.");
        }
        const results = this._archive.search(new RegExp(name + "(\.?.*)$"));
        if (results.length) {
            if (as_array_buffer === undefined) {
                as_array_buffer = SingleMediaArchive.autoDetectIfArrayBuffer();
            }
            return this._archive.read(results[0], as_array_buffer ? "arraybuffer" : "blob");
        }
        throw new Error("File not found");
    }
    async sideload(archive) {
        this._archive = archive;
        this._ok = false;
        await (this._ready = archive.ready());
        this._ok = true;
    }
    static autoDetectIfArrayBuffer() {
        return typeof Blob === 'undefined';
    }
}
exports.SingleMediaArchive = SingleMediaArchive;
