"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const big_integer_1 = __importDefault(require("big-integer"));
// -------------------------
// - ABOUT DIRECT MESSAGES -
// -------------------------
/**
 * Iterates over events inside direct messages.
 *
 * If you want messages events to be iterated, set {include_messages} to `true`.
 */
function* getEventsFromMessages(msgs, include_messages = false) {
    function* addEvents(e) {
        for (const [key, vals] of Object.entries(e)) {
            for (const val of vals) {
                yield { [key]: val };
            }
        }
    }
    let first = true;
    for (const msg of msgs) {
        if (first) {
            first = false;
            if (msg.events && msg.events.before) {
                yield* addEvents(msg.events.before);
            }
        }
        if (include_messages) {
            yield { messageCreate: msg };
        }
        if (msg.events && msg.events.after) {
            yield* addEvents(msg.events.after);
        }
    }
}
exports.getEventsFromMessages = getEventsFromMessages;
// ---------------
// - ABOUT DATES -
// ---------------
/**
 * Parse a raw Twitter date, like from a `dm.createdAt`.
 *
 * For a tweet, please use `TwitterHelpers.dateFromTweet(tweet)` instead, it's optimized !
 *
 * For a `LinkedDirectMessage`, use property `.createdAtDate` !
 */
function parseAdDate(date) {
    return new Date(date.split(" ", 2).join("T") + ".000Z");
}
exports.parseAdDate = parseAdDate;
/**
 * Parse a raw Twitter date, from a `dm.createdAt` or `tweet.created_at`.
 *
 * For a tweet, please use `TwitterHelpers.dateFromTweet(tweet)` instead, it's optimized !
 *
 * For dates in ad data (everthing that comes from `AdArchive`), use instead `TwitterHelpers.parseAdDate()` !
 *
 * For a `LinkedDirectMessage`, use property `.createdAtDate` !
 */
function parseTwitterDate(date) {
    try {
        const d = new Date(date);
        if (isNaN(d.getTime())) {
            throw "";
        }
        else {
            return d;
        }
    }
    catch (e) {
        return new Date(date.replace(/\s(.+)\s.+/, 'T$1.000Z'));
    }
}
exports.parseTwitterDate = parseTwitterDate;
/** Return the `Date` object affiliated to **tweet**. */
function dateFromTweet(tweet) {
    if (tweet.created_at_d && tweet.created_at_d instanceof Date) {
        return tweet.created_at_d;
    }
    return tweet.created_at_d = parseTwitterDate(tweet.created_at);
}
exports.dateFromTweet = dateFromTweet;
/**
 * Parse a date inside a `PushDevice` or a `MessagingDevice` object.
 *
 * Should not be used for any other type of object !
 */
function parseDeviceDate(date) {
    try {
        const d = new Date(date);
        // Chrome can parse YYYY.MM.DD correctly...
        if (!isNaN(d.getTime())) {
            return d;
        }
    }
    catch (e) { }
    return new Date(date.replace(/\./g, '-'));
}
exports.parseDeviceDate = parseDeviceDate;
// ----------------
// - ABOUT TWEETS -
// ----------------
/**
 * Sort tweets by ID (descending order by default).
  */
function sortTweets(tweets, order = "desc") {
    let sort_fn;
    if (helpers_1.supportsBigInt()) {
        if (order === "asc")
            sort_fn = (a, b) => Number(BigInt(a.id_str) - BigInt(b.id_str));
        else
            sort_fn = (a, b) => Number(BigInt(b.id_str) - BigInt(a.id_str));
    }
    else {
        if (order === "asc")
            sort_fn = (a, b) => (big_integer_1.default(a.id_str).minus(big_integer_1.default(b.id_str))).toJSNumber();
        else
            sort_fn = (a, b) => (big_integer_1.default(b.id_str).minus(big_integer_1.default(a.id_str))).toJSNumber();
    }
    return tweets.sort(sort_fn);
}
exports.sortTweets = sortTweets;
/**
 * True if given tweet is coming from a GDPR archive.
 *
 * Tweets getted by available getters are NOT GDPR tweets, they've been converted !
 */
function isGDPRTweet(tweet) {
    return 'retweet_count' in tweet;
}
exports.isGDPRTweet = isGDPRTweet;
/**
 * Return true if **tweet** contains media(s).
 *
 * This includes photos, videos or animated GIF.
 */
function isWithMedia(tweet) {
    return tweet.entities.media.length > 0;
}
exports.isWithMedia = isWithMedia;
/**
 * Return true if **tweet** contains a video or one animated GIF.
 *
 * Twitter's GIF are mp4 encoded.
 */
function isWithVideo(tweet) {
    if (tweet.extended_entities) {
        if (tweet.extended_entities.media) {
            return tweet.extended_entities.media.some(m => m.type !== "photo");
        }
    }
    return false;
}
exports.isWithVideo = isWithVideo;
